#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "file_open.h"
static char *s_flag = "cmp";

static int compare(FILE *fp1, FILE *fp2, const char *s1, const char *s2) {
	if (fp1 == fp2)
		return 0;

	int ch1;
	int ch2;

	size_t byte = 1;
	size_t lines = 1;
	do {
		ch1 = getc(fp1);
		ch2 = getc(fp2);

		if (ch1 == EOF) {
			printf("EOF on %s\n", s1);
			return 1;
		}

		else if (ch2 == EOF) {
			printf("EOF on %s\n", s2);
			return 1;
		}

		else if (ch1 != ch2) {
			if (s_flag)
				printf("files differ at byte %zu and line %zu\n", byte, lines);

			return 1;
		}

		if (ch1 == '\n')
			lines++;

		byte++;
	} while(ch1 != EOF);

	return 0;
}

static long parse_int(const char *str) {
	char *ptr;
	long val = strtol(str, &ptr, 0);
	if (*ptr || val < 0) {
		fprintf(stderr, "cmp: invalid offset: %s\n", str);
		exit(1);
	}

	return val;
}

int main(int argc, char **argv) {

	int opt;
	while ((opt = getopt(argc, argv, "s")) != -1) {
		switch (opt) {
			case 's':
				s_flag = NULL;
				break;

			default:
				puts("cmp [s] [file1] [file2] [skip1] [skip2]\n\t-s Silent");
				return 0;
		}
	}


	argv += optind;
	argc -= optind;

	long skip1 = 0;
	long skip2 = 0;
	FILE *fp1 = NULL;
	FILE *fp2 = stdin;

	char *s1 = "-";
	char *s2 = "-";

	switch (argc) {
		case 4:
			skip2 = parse_int(argv[3]);
			/* fallthrough */

		case 3:
			skip1 = parse_int(argv[2]);
			/* fallthrough */

		case 2:
			s2 = argv[1];
			fp2 = file_open(s_flag, argv[1], "r");
			if (fp2 == NULL)
				exit(1);

			/* fallthrough */

		case 1:
			s1 = argv[0];
			fp1 = file_open(s_flag, argv[0], "r");
			if (fp1 == NULL) {
				fclose(fp2);
				exit(1);
			}

			break;

		default:
			fprintf(stderr, "cmp: missing operand\n");
			return 1;
	}

	if (skip1 && fseek(fp1, skip1, SEEK_SET) < 0) {
		if (s_flag)
			fprintf(stderr, "cmp: %s\n", strerror(errno));

		return 1;
	}

	if (skip2 && fseek(fp2, skip2, SEEK_SET) < 0) {
		if (s_flag)
			fprintf(stderr, "cmp: %s\n", strerror(errno));

		return 1;
	}

	int ret = compare(fp1, fp2, s1, s2);
	fclose(fp1);
	fclose(fp2);

	return ret;
}
